import os
import es
import gamethread

info = es.AddonInfo()
info.name = 'Synergy Map Tracker'
info.basename = 'syn_maptracker'
info.version = '1.3'
info.author = 'GODJonez'

class PathHelper(object):
    def __init__(self, basepath):
        self.basepath = basepath
    def __add__(self, sub):
        return os.path.join(self.basepath, sub)
    def __str__(self):
        return os.path.normpath(self.basepath)
    def __repr__(self):
        return os.path.normpath(self.basepath)

class MapInfo(object):
    def __init__(self, mapname):
        self.mapname = mapname
        self.played = False
        self.isnew = True
        self.info = ''
        self.deleted = False
    def __str__(self):
        return self.mapname
    def filename(self):
        return self.mapname+'.bsp'
    def __repr__(self):
        if self.deleted:
            return '-'+str(self)+self.info
        if self.played:
            return '*'+str(self)+self.info
        elif self.isnew:
            return '+'+str(self)+self.info
        else:
            return ' '+str(self)+self.info

class MapList(dict):
    def get_all(self):
        return map(lambda x: repr(x[1]), sorted(self.items()))
    def get_unseen(self):
        allmaps = self.get_all()
        return filter(lambda x: x[0] != '*',allmaps)
    def get_new(self):
        allmaps = self.get_all()
        return filter(lambda x: x[0] == '+',allmaps)
    def next_unseen(self):
        fallback = ''
        for mapinfo in self.values():
            if not mapinfo.played and not mapinfo.deleted and 'noauto' not in mapinfo.info:
                if 'mess' in mapinfo.info:
                    if not fallback:
                        fallback = str(mapinfo)
                else:
                    return str(mapinfo)
        return fallback

addonpath = PathHelper(es.getAddonPath(info.basename))
maplist = MapList()
thismap = es.ServerVar('eventscripts_currentmap')
nextmap = es.ServerVar('eventscripts_nextmapoverride')
autogo = es.ServerVar('syn_maptracker_go', '1', 'Automatically set next map to be one of the unplayed ones')

def load():
    maplist.clear()
    allmaps = os.listdir(os.path.join(str(es.ServerVar('eventscripts_gamedir')), 'maps'))
    for mapname in allmaps:
        if mapname.endswith('.bsp'):
            rname = mapname[:-4]
            maplist[rname] = MapInfo(rname)

    load_old_map_list()
    save_new_map_list()

    es.addons.registerClientCommandFilter(ccf)
    if not es.exists('command', 'synmap_start'):
        es.regcmd('synmap_start', info.basename+'/startcmd')
        es.regcmd('synmap_next', info.basename+'/nextcmd')
        es.regcmd('synmaps', info.basename+'/synmaps_cmd')

def unload():
    es.addons.unregisterClientCommandFilter(ccf)

def startcmd():
    nm = maplist.next_unseen()
    es.server.cmd('map %s'%nm)

def nextcmd():
    nm = maplist.next_unseen()
    es.server.queuecmd('changelevel %s'%nm)

def synmaps_cmd():
    args = (es.getargs() or '').split()
    get_synmaps(args, es.dbgmsg, (0,))

def get_synmaps(args, cmd, params):
        if not args:
            cmd(*(params + ('Usage: synmaps {all|unseen|new}',)))
        elif args[0] == 'all':
            for mapr in maplist.get_all():
               cmd(*(params + (mapr,)))
        elif args[0] == 'unseen':
            for mapr in maplist.get_unseen():
                cmd(*(params + (mapr,)))
        elif args[0] == 'new':
            for mapr in maplist.get_new():
                cmd(*(params + (mapr[1:],)))

def ccf(userid, args):
    if args[0] == 'synmaps':
        get_synmaps(args[1:], es.cexec, (userid,'echo',))
        return False
    return True

def es_player_chat(ev):
    args = ev['text'].split()
    if args[0] == 'mapinfo':
        maplist[str(thismap)].info+=' '+' '.join(args[1:])
        es.tell(ev['userid'], 'Info for '+str(thismap)+': '+maplist[str(thismap)].info)
        save_new_map_list()

def es_map_start(ev):
    mapname = ev['mapname']
    mapobj = maplist[mapname]
    mapobj.played = True
    save_new_map_list()
    if int(autogo) and 'continues' not in mapobj.info:
        nm = maplist.next_unseen()
        nextmap.set(nm)
    if 'proper' in mapobj.info:
        es.set('mp_spawnsystem', 0)
    else:
        es.set('mp_spawnsystem', 1)

def player_activate(ev):
    userid = int(ev['userid'])
    gamethread.delayed(10, announce, userid)

def announce(userid):
    es.tell(userid, '\x04Synergy Map Tracker active')
    es.tell(userid, '\x01Current map: \x03%s [%s]'%(thismap,maplist[str(thismap)].info))
    es.tell(userid, '\x01Next map: \x04%s'%nextmap)

def load_old_map_list():
    try:
        f = open(addonpath+'maplist.txt','r')
        for mapname in f:
            mapinfo = mapname[1:-1].split(' ', 1)
            rname = mapinfo[0]
            if mapname[0] == '-':
                # ignore maps previously marked as deleted
                if rname not in maplist:
                    continue
                else:
                    # the map has been added again!
                    maplist[rname].deleted = False
            if rname not in maplist:
                es.dbgmsg(0, 'Map %s has been deleted from server!'%rname)
                maplist[rname] = MapInfo(rname)
                maplist[rname].deleted = True
            if mapname[0] == '*':
                maplist[rname].played = True
            maplist[rname].isnew = False
            if len(mapinfo) > 1:
                maplist[rname].info = ' '+mapinfo[1]
        f.close()
    except IOError:
        pass

def save_new_map_list():
    try:
        f = open(addonpath+'maplist.txt','w')
        for mapinfo in maplist.values():
            f.write(repr(mapinfo)+'\n')
        f.close()
    except IOError, e:
        es.dbgmsg(0, info.basename+': Could not save maplist.txt: '+str(e))
